#include <petsc.h>

int main(int argc,char **args) {
  Mat            A,B;                       /* matrix */
  PetscViewer    fd;                      /* viewer */
  char           file[PETSC_MAX_PATH_LEN];            /* input file name */
  PetscErrorCode ierr;
  PetscBool      flg;

  PetscInitialize(&argc,&args,NULL,NULL);

  int rank, size;
  MPI_Comm_size(PETSC_COMM_WORLD, &size);
  MPI_Comm_rank(PETSC_COMM_WORLD, &rank);

  if (size != 4) SETERRQ(PETSC_COMM_WORLD,1,"Must use four processes");

  /*
     Determine files from which we read the two linear systems
     (matrix and right-hand-side vector).
  */
  ierr = PetscOptionsGetString(NULL,NULL,"-f",file,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
  if (!flg) SETERRQ(PETSC_COMM_WORLD,1,"Must indicate binary file with the -f option");

  /*
     Open binary file.  Note that we use FILE_MODE_READ to indicate
     reading from this file.
  */
  ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file,FILE_MODE_READ,&fd);CHKERRQ(ierr);

  /*
     Load the matrix; then destroy the viewer.
  */
  ierr = MatCreate(PETSC_COMM_WORLD,&A);CHKERRQ(ierr);
  ierr = MatLoad(A,fd);CHKERRQ(ierr);
  ierr = PetscViewerDestroy(&fd);CHKERRQ(ierr);
  KSP ksp;
  KSPCreate(PETSC_COMM_WORLD,&ksp);
  KSPSetOperators(ksp,A,A);
  PC pc;
  KSPGetPC(ksp, &pc);
  PetscInt idx[2];
  IS is[2];
  for(int j = 0; j < 2; ++j) {
    for(int i = 0; i < 2; ++i)
      idx[i] = 2 * (rank % 2) + 4 * (rank / 2) + i;
    PetscErrorCode ierr;
    ierr = ISCreateGeneral(PETSC_COMM_WORLD, (j == 0 && rank < 2) || (j > 0 && rank >= 2) ? 2 : 0, idx, PETSC_COPY_VALUES, is + j);CHKERRQ(ierr);
    ISView(is[j], PETSC_VIEWER_STDOUT_WORLD);
    PCFieldSplitSetIS(pc, NULL, is[j]);
  }
  KSPSetFromOptions(ksp);
  KSPSetUp(ksp);
  ISDestroy(is);
  ISDestroy(is + 1);
  KSPDestroy(&ksp);
  MatDestroy(&A);
  ierr = PetscFinalize();
  return 0;
}
